#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cJSON.h"

// 假设的API函数，获取玩家和敌人的位置
void getPlayerPosition(float *x, float *y, float *z);
void getEnemyPosition(float *x, float *y, float *z);
void setPlayerRotation(float yaw, float pitch);

// 读取JSON文件
void readJsonFile(const char *filename, float *initialSpeed, float *gravity, float *dragCoefficient, float *maxLength, float *minLength, int *lockOn, float *lockOnRange, float *lockOnSpeed) {
    FILE *file = fopen(filename, "r");
    if (file == NULL) {
        printf("Failed to open file: %s\n", filename);
        return;
    }

    fseek(file, 0, SEEK_END);
    long length = ftell(file);
    fseek(file, 0, SEEK_SET);

    char *buffer = malloc(length + 1);
    fread(buffer, 1, length, file);
    buffer[length] = '\0';

    cJSON *json = cJSON_Parse(buffer);
    if (json == NULL) {
        printf("Failed to parse JSON: %s\n", cJSON_GetErrorPtr());
        free(buffer);
        fclose(file);
        return;
    }

    cJSON *bullet = cJSON_GetObjectItemCaseSensitive(json, "Bullet");
    if (cJSON_IsObject(bullet)) {
        *initialSpeed = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "InitialSpeed"));
        *gravity = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "Gravity"));
        *dragCoefficient = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "DragCoefficient"));
        *maxLength = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "MaxLength"));
        *minLength = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "MinLength"));
        *lockOn = cJSON_IsTrue(cJSON_GetObjectItemCaseSensitive(bullet, "LockOn"));
        *lockOnRange = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "LockOnRange"));
        *lockOnSpeed = cJSON_GetNumberValue(cJSON_GetObjectItemCaseSensitive(bullet, "LockOnSpeed"));
    }

    cJSON_Delete(json);
    free(buffer);
    fclose(file);
}

// 计算两个点之间的方向向量
void calculateDirection(float x1, float y1, float z1, float x2, float y2, float z2, float *dx, float *dy, float *dz) {
    *dx = x2 - x1;
    *dy = y2 - y1;
    *dz = z2 - z1;
}

// 将方向向量转换为旋转角度
void directionToRotation(float dx, float dy, float dz, float *yaw, float *pitch) {
    *yaw = atan2(dx, dz) * (180.0 / M_PI); // Yaw (左右旋转)
    *pitch = atan2(dy, sqrt(dx * dx + dz * dz)) * (180.0 / M_PI); // Pitch (上下旋转)
}

// 主函数，实现子弹追踪
void trackBullet(float initialSpeed, float gravity, float dragCoefficient, float maxLength, float minLength, int lockOn, float lockOnRange, float lockOnSpeed) {
    float playerX, playerY, playerZ;
    float enemyX, enemyY, enemyZ;
    float dx, dy, dz;
    float yaw, pitch;

    // 获取玩家和敌人的位置
    getPlayerPosition(&playerX, &playerY, &playerZ);
    getEnemyPosition(&enemyX, &enemyY, &enemyZ);

    // 计算方向向量
    calculateDirection(playerX, playerY, playerZ, enemyX, enemyY, enemyZ, &dx, &dy, &dz);

    // 将方向向量转换为旋转角度
    directionToRotation(dx, dy, dz, &yaw, &pitch);

    // 设置玩家的旋转角度
    setPlayerRotation(yaw, pitch);

    // 打印调试信息
    printf("Player Position: (%f, %f, %f)\n", playerX, playerY, playerZ);
    printf("Enemy Position: (%f, %f, %f)\n", enemyX, enemyY, enemyZ);
    printf("Yaw: %f, Pitch: %f\n", yaw, pitch);
}

int main() {
    float initialSpeed, gravity, dragCoefficient, maxLength, minLength;
    int lockOn;
    float lockOnRange, lockOnSpeed;

    // 读取JSON文件
    readJsonFile("bullet_config.json", &initialSpeed, &gravity, &dragCoefficient, &maxLength, &minLength, &lockOn, &lockOnRange, &lockOnSpeed);

    // 模拟游戏循环
    while (1) {
        if (lockOn) {
            trackBullet(initialSpeed, gravity, dragCoefficient, maxLength, minLength, lockOn, lockOnRange, lockOnSpeed);
        }
        // 模拟游戏循环的延迟
        sleep(1);
    }
    return 0;
}
